function [distance, pval2t, Z, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dcompare(W1,W2,D,tripletabc,CItype,init)

%INPUT ARGUMENTS:
%W1: A column vector of biomarker measurements taken with biomarker 1.
%W2: A column vector of biomarker measurements taken with biomarker 2.
%D: The disease status. A column vector taking values 1,2 or 3 depending on whether the $i-th$ individual belongs to group A, B or C respectively.
%tripletABC: A row vector of three elements that will indicate the given pair at which the comparison will take place. This vector should contain exactly a \texttt{NaN} (not a number) value. For example, if the given pair is $TC_{A}=0.5$ and $TC_{C}=0.6$ then this input argument should be set as \texttt{[0.5 NaN 0.6]}. Analogously, if the given pair is $TC_{B}=0.5$ and $TC_{C}=0.6$ then this input argument should be set as \texttt{[NaN 0.5 0.6]}. Similarly, for the case where $TC_{A}=0.5$ and $TC_{B}=0.6$ are given then this input argument should be set as \texttt{[0.5 0.6 NaN]}.
%CItype: The method employed. This input argument can be set as \texttt{normal}, or  \texttt{boxcox}, or \texttt{kernels} depending on the approach that the user want to be employed.
%init: This input refers only to the Box-Cox based approach and can also not be reached. It refers to initial values of the parameters involved in the likelihood function of the Box-Cox approach $(\mu_{1A}, \sigma_{1A}, \mu_{1B}, \sigma_{1B}, \mu_{1C}, \sigma_{1C}, \mu_{2A}, \sigma_{2A}, \mu_{2B}, \sigma_{2B}, \mu_{2C}, \sigma_{2C}, cov_{A}, cov_{B}, cov_{C}, \lambda_{1}, \lambda_{2})$. If this argument is not reached the sample means and standard deviations are employed along with the values 0.5 and 1 for all the covariances and transformation parameters ($\lambda_{1}$ and $\lambda_{2}$) respectively.

%OUTPUT ARGUMENTS:
%distance: The distance between the two ROC surfaces at the given pair of true classification rates.
%pval2t: A two tailed $p-value$ regarding the targeted comparison based on statistic $Z^{*}$.
%Z: The actual value of the statistic $Z^{*}$.
%ROC1: The estimated point on the ROC surface $(\hat{ROC}_{1}(TC_{i},TC_{k}))$ that corresponds to biomarker 1, where $i,j=A,B,C$ with $i\neq j$. The estimated point is based on the input argument \texttt{tripletABC} given by the user.
%ROC2: The estimated point on the ROC surface $(\hat{ROC}_{2}(TC_{i},TC_{k}))$ that corresponds to biomarker 2, where $i,j=A,B,C$ with $i\neq j$. The estimated point is based on the input argument \texttt{tripletABC} given by the user.
%vus1: Volume under the surface of biomarker 1.
%vus2: Volume under the surface of biomarker 2.
%CIdiff: $95\%$ confidence interval for the difference $\Phi(\hat{ROC}_{1}(TC_{i},TC_{k}))-\Phi(\hat{ROC}_{2}(TC_{i},TC_{k}))$, , where $i,j=A,B,C$ with $i\neq j$.

%Apart from the above output arguments two figures automatically appear that visualize the comparison
%of the two surfaces as well as a depiction of the pairwise ROC curves. Furthermore the feasible region
%of a given true classification rate is also provided by the second figure. 
%The 3D figure is dynamic and can be rotated by the user at any angle for better visualization. 

%======================================================================================================
%======================================================================================================
%The code is also provided by the journal's website as well as in the webpage of the first author.
%www.leobantis.net
%======================================================================================================
%======================================================================================================


if size(W1,1)==1;W1=W1';end
if size(W2,1)==1;W2=W2';end
if size(D,1)==1;D=D';end

if length(unique(D))~=3;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
if min(D)~=1;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
if max(D)~=3;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end

aD=unique(D);aD=sort(aD);
if length(aD)~=3;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
if aD(1)~=1;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
if aD(2)~=2;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
if aD(3)~=3;error('D should indicate three groups 1,2 and 3 for separation, higher D more is more indicative of the disease');end
 

na=length(W1(D==1));
nb=length(W1(D==2));
nc=length(W1(D==3));

if length(W2(D==1))~=na;error('Number of individuals of the two markers must be the same (check group A)');end
if length(W2(D==2))~=nb;error('Number of individuals of the two markers must be the same (check group B)');end
if length(W2(D==3))~=nc;error('Number of individuals of the two markers must be the same (check group C)');end


if length(find(isnan(tripletabc)))~=1
    error('the tripletabc should have exactly one NaN value so that a pair is defined')
end

switch CItype
    case 'normal'  
        if find(isnan(tripletabc))==1
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dnormal_bc(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==2
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dnormal_ac(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==3
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dnormal_ab(W1,W2,D,tripletabc);
        end
        Z=Zstar;
        
    case 'boxcox'
        if nargin==5
            
        if find(isnan(tripletabc))==1
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_bc(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==2
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_ac(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==3
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_ab(W1,W2,D,tripletabc);
        end
        Z=Zstar;
        
        elseif nargin==6
        if find(isnan(tripletabc))==1
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_bc(W1,W2,D,tripletabc,init);
        elseif find(isnan(tripletabc))==2
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_ac(W1,W2,D,tripletabc,init);
        elseif find(isnan(tripletabc))==3
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dboxcox_ab(W1,W2,D,tripletabc,init);
        end
        Z=Zstar;
        
        end
            
            
        
    case 'kernels'
        
        if find(isnan(tripletabc))==1
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dkernel_bc(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==2
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dkernel_ac(W1,W2,D,tripletabc);
        elseif find(isnan(tripletabc))==3
            [distance, pval2t, Zstar, ROC1, ROC2, vus1, vus2, Cidiff]=roc3dkernel_ab(W1,W2,D,tripletabc);
        end
        Z=Zstar;
        
        
        
        

end

end
    
